@@ -85,7 +85,7 @@ reclass_date.ti <- function(new, orig) {
 }
 #' @export
 reclass_date.Date <- function(new, orig) {
-  as.Date(new)
+  as_date(new)
 }
 
 
@@ -601,12 +601,20 @@ setMethod("as.character", signature(x = "Interval"), function(x, ...) {
 #' tweaks to make them work more intuitively.
 #'
 #' \itemize{
-#'   \item `as_date()` ignores the timezone attribute, resulting in
-#'      a more intuitive conversion (see examples)
-#'   \item Both functions provide a default origin argument for numeric
-#'      vectors.
-#'   \item Both functions will generate NAs for invalid date format. A warning message will provide a count of the elements that were not converted
-#'   \item `as_datetime()` defaults to using UTC.
+#'
+#' \item Called on a `POSIXct` object, `as_date()` uses the tzone attribute of
+#'   the object to return the same date as indicated by the printed representation
+#'   of the object. This differs from as.Date, which ignores the attribute and
+#'   uses only the tz argument to `as.Date()` ("UTC" by default).
+#'
+#' \item Both functions provide a default origin argument for numeric vectors.
+#'
+#' \item Both functions will generate NAs for
+#'   invalid date format. A warning message will provide a count of the elements
+#'   that were not converted
+#'
+#' \item `as_datetime()` defaults to using UTC.
+#'
 #' }
 #'
 #' @param x a vector of [POSIXt], numeric or character objects
@@ -710,16 +718,29 @@ setMethod("as_datetime", "numeric",
 #' @rdname as_date
 #' @export
 setMethod("as_datetime", "character",
-          function(x, tz = "UTC", format = NULL) {
-            if (is.null(format))
-              .parse_iso_dt(x, tz)
-            else
-              as.POSIXct(strptime(x, format = format, tz = tz))
-          })
+  function(x, tz = "UTC", format = NULL) {
+    if (is.null(format))
+      .parse_iso_dt(x, tz)
+    else
+      as.POSIXct(strptime(x, format = format, tz = tz))
+  })
+
+
+#' @rdname as_date
+#' @export
+setMethod("as_datetime", "Date",
+  function(x, tz = "UTC") {
+    dt <- .POSIXct(as.numeric(x) * 86400, tz = "UTC")
+    if (is_utc(tz))
+      return(dt)
+    else
+      force_tz(dt, tzone = tz)
+  })
+
 
 #' @rdname as_date
 #' @export
 setMethod("as_datetime", "ANY",
   function(x, tz = "UTC") {
-    with_tz(as.POSIXct(x), tzone = tz)
+    with_tz(as.POSIXct(x, tz = tz), tzone = tz)
   })
